// Generated by CoffeeScript 1.12.7
(function() {
  var ExpressHandler, Raven, STATUS_CODES, _, debug, errorhandler,
    bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  _ = require('lodash');

  errorhandler = require('errorhandler');

  Raven = require('raven');

  STATUS_CODES = require('http').STATUS_CODES;

  debug = require('debug')('octoblu-raven:express');

  ExpressHandler = (function() {
    function ExpressHandler(arg) {
      var ref;
      ref = arg != null ? arg : {}, this.dsn = ref.dsn, this.logFn = ref.logFn;
      this._fakeIt = bind(this._fakeIt, this);
      this._getCode = bind(this._getCode, this);
      this._getMessage = bind(this._getMessage, this);
      this._getResponseMessage = bind(this._getResponseMessage, this);
      this._getErrorWithCode = bind(this._getErrorWithCode, this);
      this._getError = bind(this._getError, this);
      this._respondWithError = bind(this._respondWithError, this);
      this._logError = bind(this._logError, this);
      this._sendErrorHandler = bind(this._sendErrorHandler, this);
      this.sendErrorHandler = bind(this.sendErrorHandler, this);
      this.requestHandler = bind(this.requestHandler, this);
      this.errorHandler = bind(this.errorHandler, this);
    }

    ExpressHandler.prototype.errorHandler = function() {
      debug('errorHandler');
      if (this.dsn == null) {
        return errorhandler();
      }
      return Raven.errorHandler();
    };

    ExpressHandler.prototype.requestHandler = function() {
      debug('requestHandler');
      if (this.dsn == null) {
        return this._fakeIt();
      }
      return Raven.requestHandler();
    };

    ExpressHandler.prototype.sendErrorHandler = function() {
      debug('sendErrorHandler');
      return this._sendErrorHandler;
    };

    ExpressHandler.prototype._sendErrorHandler = function(request, response, next) {
      debug('_sendErrorHandler');
      response.sendError = (function(_this) {
        return function(error, code) {
          if (code == null) {
            code = _this._getCode(error);
          }
          _this._logError(error, code);
          return _this._respondWithError(response, error, code);
        };
      })(this);
      return next();
    };

    ExpressHandler.prototype._logError = function(error, code) {
      var message;
      message = _.get(error, 'stack', error);
      if (code < 500) {
        return debug(message);
      }
      return this.logFn(message);
    };

    ExpressHandler.prototype._respondWithError = function(response, error, code) {
      return response.status(code).send(this._getResponseMessage(error, code));
    };

    ExpressHandler.prototype._getError = function(error) {
      if (error == null) {
        return new Error('[octoblu-raven] sendError called without an error');
      }
      if (_.isError(error)) {
        return error;
      }
      return new Error(this._getMessage(error));
    };

    ExpressHandler.prototype._getErrorWithCode = function(error) {
      error = this._getError(error);
      error.code = this._getCode(error);
      return error;
    };

    ExpressHandler.prototype._getResponseMessage = function(error, code) {
      var message;
      if (!_.isError(error) && _.isPlainObject(error)) {
        return error;
      }
      message = this._getMessage(error, code);
      if (!message) {
        return STATUS_CODES[code];
      }
      return {
        error: message
      };
    };

    ExpressHandler.prototype._getMessage = function(error) {
      if (_.isString(error)) {
        return error;
      }
      if ((error != null ? error.message : void 0) === '[object Object]') {
        return;
      }
      if (!(error != null ? error.message : void 0)) {
        return;
      }
      return error.message;
    };

    ExpressHandler.prototype._getCode = function(error) {
      var code;
      code = _.get(error, 'code');
      try {
        code = _.toNumber(code);
      } catch (error1) {}
      if (STATUS_CODES[code] != null) {
        return error.code;
      }
      return 500;
    };

    ExpressHandler.prototype._fakeIt = function() {
      return (function(_this) {
        return function(request, response, next) {
          return next();
        };
      })(this);
    };

    return ExpressHandler;

  })();

  module.exports = ExpressHandler;

}).call(this);

//# sourceMappingURL=express-handler.js.map
